@@ -1,4 +1,5 @@
 /*
+ * The two pass scaling function is based on:
  * Filtered Image Rescaling
  * Based on Gems III
  *  - Schumacher general filtered image rescaling
@@ -13,6 +14,7 @@
  *
  * 	Initial sources code is avaibable in the Gems Source Code Packages:
  * 	http://www.acm.org/pubs/tog/GraphicsGems/GGemsIII.tar.gz
+ *
  */
 
 /*
@@ -39,8 +41,8 @@
 	downscaling using the fixed point implementations are usually much faster
 	than the existing gdImageCopyResampled while having a similar or better
 	quality.
-
-	For image rotations, the optimized versions have a lazy antialiasing for
+	
+	For image rotations, the optimized versions have a lazy antialiasing for 
 	the edges of the images. For a much better antialiased result, the affine
 	function is recommended.
 */
@@ -633,7 +635,7 @@ static inline int _color_blend (const int dst, const int src)
 	}
 }
 
-static inline int _setEdgePixel(const gdImagePtr src, unsigned int x, unsigned int y, gdFixed coverage, const int bgColor)
+static inline int _setEdgePixel(const gdImagePtr src, unsigned int x, unsigned int y, gdFixed coverage, const int bgColor) 
 {
 	const gdFixed f_127 = gd_itofx(127);
 	register int c = src->tpixels[y][x];
@@ -816,10 +818,6 @@ int getPixelInterpolated(gdImagePtr im, const double x, const double y, const in
 		return -1;
 	}
 
-	/* Default to full alpha */
-	if (bgColor == -1) {
-	}
-
 	if (im->interpolation_id == GD_WEIGHTED4) {
 		return getPixelInterpolateWeight(im, x, y, bgColor);
 	}
@@ -881,41 +879,20 @@ int getPixelInterpolated(gdImagePtr im, const double x, const double y, const in
 static inline LineContribType * _gdContributionsAlloc(unsigned int line_length, unsigned int windows_size)
 {
 	unsigned int u = 0;
-	LineContribType *res;
-	int overflow_error = 0;
+    LineContribType *res;
 
 	res = (LineContribType *) gdMalloc(sizeof(LineContribType));
 	if (!res) {
 		return NULL;
 	}
-	res->WindowSize = windows_size;
-	res->LineLength = line_length;
-	if (overflow2(line_length, sizeof(ContributionType))) {
-		gdFree(res);
-		return NULL;
-	}
-	res->ContribRow = (ContributionType *) gdMalloc(line_length * sizeof(ContributionType));
-	if (res->ContribRow == NULL) {
-		gdFree(res);
-		return NULL;
-	}
-	for (u = 0 ; u < line_length ; u++) {
-		if (overflow2(windows_size, sizeof(double))) {
-			overflow_error = 1;
-		} else {
-			res->ContribRow[u].Weights = (double *) gdMalloc(windows_size * sizeof(double));
-		}
-		if (overflow_error == 1 || res->ContribRow[u].Weights == NULL) {
-			unsigned int i;
-			u--;
-			for (i=0;i<=u;i++) {
-				gdFree(res->ContribRow[i].Weights);
-			}
-			gdFree(res);
-			return NULL;
-		}
-	}
-	return res;
+    res->WindowSize = windows_size;
+    res->LineLength = line_length;
+    res->ContribRow = (ContributionType *) gdMalloc(line_length * sizeof(ContributionType));
+
+    for (u = 0 ; u < line_length ; u++) {
+        res->ContribRow[u].Weights = (double *) gdMalloc(windows_size * sizeof(double));
+    }
+    return res;
 }
 
 static inline void _gdContributionsFree(LineContribType * p)
@@ -930,62 +907,59 @@ static inline void _gdContributionsFree(LineContribType * p)
 
 static inline LineContribType *_gdContributionsCalc(unsigned int line_size, unsigned int src_size, double scale_d,  const interpolation_method pFilter)
 {
-	double width_d;
-	double scale_f_d = 1.0;
-	const double filter_width_d = DEFAULT_BOX_RADIUS;
+    double width_d;
+    double scale_f_d = 1.0;
+    const double filter_width_d = DEFAULT_BOX_RADIUS;
 	int windows_size;
 	unsigned int u;
 	LineContribType *res;
-	int overflow_error = 0;
 
-	if (scale_d < 1.0) {
-		width_d = filter_width_d / scale_d;
-		scale_f_d = scale_d;
-	}  else {
-		width_d= filter_width_d;
-	}
+    if (scale_d < 1.0) {
+        width_d = filter_width_d / scale_d;
+        scale_f_d = scale_d;
+    }  else {
+        width_d= filter_width_d;
+    }
 
-	windows_size = 2 * (int)ceil(width_d) + 1;
-	res = _gdContributionsAlloc(line_size, windows_size);
-	if (res == NULL) {
-		return NULL;
-	}
-	for (u = 0; u < line_size; u++) {
-	const double dCenter = (double)u / scale_d;
-	/* get the significant edge points affecting the pixel */
-	register int iLeft = MAX(0, (int)floor (dCenter - width_d));
-	int iRight = MIN((int)ceil(dCenter + width_d), (int)src_size - 1);
-	double dTotalWeight = 0.0;
+    windows_size = 2 * (int)ceil(width_d) + 1;
+    res = _gdContributionsAlloc(line_size, windows_size);
+
+    for (u = 0; u < line_size; u++) {
+        const double dCenter = (double)u / scale_d;
+        /* get the significant edge points affecting the pixel */
+        register int iLeft = MAX(0, (int)floor (dCenter - width_d));
+        int iRight = MIN((int)ceil(dCenter + width_d), (int)src_size - 1);
+        double dTotalWeight = 0.0;
 		int iSrc;
 
-	/* Cut edge points to fit in filter window in case of spill-off */
-	if (iRight - iLeft + 1 > windows_size)  {
-		if (iLeft < ((int)src_size - 1 / 2))  {
-			iLeft++;
-		} else {
-			iRight--;
-		}
-	}
+        res->ContribRow[u].Left = iLeft;
+        res->ContribRow[u].Right = iRight;
 
-	res->ContribRow[u].Left = iLeft;
-	res->ContribRow[u].Right = iRight;
+        /* Cut edge points to fit in filter window in case of spill-off */
+        if (iRight - iLeft + 1 > windows_size)  {
+            if (iLeft < ((int)src_size - 1 / 2))  {
+                iLeft++;
+            } else {
+                iRight--;
+            }
+        }
 
-	for (iSrc = iLeft; iSrc <= iRight; iSrc++) {
-		dTotalWeight += (res->ContribRow[u].Weights[iSrc-iLeft] =  scale_f_d * (*pFilter)(scale_f_d * (dCenter - (double)iSrc)));
-	}
+        for (iSrc = iLeft; iSrc <= iRight; iSrc++) {
+            dTotalWeight += (res->ContribRow[u].Weights[iSrc-iLeft] =  scale_f_d * (*pFilter)(scale_f_d * (dCenter - (double)iSrc)));
+        }
 
 		if (dTotalWeight < 0.0) {
 			_gdContributionsFree(res);
 			return NULL;
 		}
 
-	if (dTotalWeight > 0.0) {
-		for (iSrc = iLeft; iSrc <= iRight; iSrc++) {
-			res->ContribRow[u].Weights[iSrc-iLeft] /= dTotalWeight;
-		}
-	}
-	}
-	return res;
+        if (dTotalWeight > 0.0) {
+            for (iSrc = iLeft; iSrc <= iRight; iSrc++) {
+                res->ContribRow[u].Weights[iSrc-iLeft] /= dTotalWeight;
+            }
+        }
+   }
+   return res;
 }
 
 static inline void _gdScaleRow(gdImagePtr pSrc,  unsigned int src_width, gdImagePtr dst, unsigned int dst_width, unsigned int row, LineContribType *contrib)
@@ -1073,9 +1047,6 @@ static inline void _gdScaleVert (const gdImagePtr pSrc, const unsigned int src_w
 	}
 
 	contrib = _gdContributionsCalc(dst_height, src_height, (double)(dst_height) / (double)(src_height), pSrc->interpolation);
-	if (contrib == NULL) {
-		return;
-	}
 	/* scale each column */
 	for (u = 0; u < dst_width - 1; u++) {
 		_gdScaleCol(pSrc, src_width, pDst, dst_width, dst_height, u, contrib);
@@ -1088,6 +1059,11 @@ gdImagePtr gdImageScaleTwoPass(const gdImagePtr src, const unsigned int src_widt
 	gdImagePtr tmp_im;
 	gdImagePtr dst;
 
+	/* Convert to truecolor if it isn't; this code requires it. */
+	if (!src->trueColor) {
+		gdImagePaletteToTrueColor(src);
+	}
+
 	tmp_im = gdImageCreateTrueColor(new_width, src_height);
 	if (tmp_im == NULL) {
 		return NULL;
@@ -1120,7 +1096,7 @@ gdImagePtr Scale(const gdImagePtr src, const unsigned int src_width, const unsig
 	_gdScaleHoriz(src, src_width, src_height, tmp_im, new_width, src_height);
 	_gdScaleVert(tmp_im, new_width, src_height, dst, new_width, new_height);
 
-	gdImageDestroy(tmp_im);
+	gdFree(tmp_im);
 	return dst;
 }
 
@@ -1249,13 +1225,7 @@ static gdImagePtr gdImageScaleBilinearPalette(gdImagePtr im, const unsigned int
 	if (new_img == NULL) {
 		return NULL;
 	}
-
-	if (transparent < 0) {
-		/* uninitialized */
-		new_img->transparent = -1;
-	} else {
-		new_img->transparent = gdTrueColorAlpha(im->red[transparent], im->green[transparent], im->blue[transparent], im->alpha[transparent]);
-	}
+	new_img->transparent = gdTrueColorAlpha(im->red[transparent], im->green[transparent], im->blue[transparent], im->alpha[transparent]);
 
 	for (i=0; i < _height; i++) {
 		long j;
@@ -1744,6 +1714,7 @@ gdImagePtr gdImageRotateNearestNeighbour(gdImagePtr src, const float degrees, co
 gdImagePtr gdImageRotateGeneric(gdImagePtr src, const float degrees, const int bgColor)
 {
 	float _angle = ((float) (-degrees / 180.0f) * (float)M_PI);
+	const int angle_rounded = (int)floor(degrees * 100);
 	const int src_w  = gdImageSX(src);
 	const int src_h = gdImageSY(src);
 	const unsigned int new_width = (unsigned int)(abs((int)(src_w * cos(_angle))) + abs((int)(src_h * sin(_angle))) + 0.5f);
@@ -1766,6 +1737,10 @@ gdImagePtr gdImageRotateGeneric(gdImagePtr src, const float degrees, const int b
 						: 0;
 
 
+	if (bgColor < 0) {
+		return NULL;
+	}
+
 	dst = gdImageCreateTrueColor(new_width, new_height);
 	if (!dst) {
 		return NULL;
@@ -2203,10 +2178,13 @@ gdImagePtr gdImageRotateInterpolated(const gdImagePtr src, const float angle, in
 
 	/* no interpolation needed here */
 	switch (angle_rounded) {
-		case 9000:
+		case -27000:
+		case   9000:
 			return gdImageRotate90(src, 0);
-		case 18000:
+		case -18000:
+		case  18000:
 			return gdImageRotate180(src, 0);
+		case -9000:
 		case 27000:
 			return gdImageRotate270(src, 0);
 	}
@@ -2306,7 +2284,7 @@ int gdTransformAffineGetImage(gdImagePtr *dst,
 	if (!src->trueColor) {
 		gdImagePaletteToTrueColor(src);
 	}
-
+	
 	/* Translate to dst origin (0,0) */
 	gdAffineTranslate(m, -bbox.x, -bbox.y);
 	gdAffineConcat(m, affine, m);
@@ -2365,7 +2343,7 @@ int gdTransformAffineCopy(gdImagePtr dst,
 	if (src->interpolation_id == GD_BILINEAR_FIXED || src->interpolation_id == GD_BICUBIC_FIXED || src->interpolation_id == GD_NEAREST_NEIGHBOUR) {
 		interpolation_id_bak = src->interpolation_id;
 		interpolation_bak = src->interpolation;
-
+		
 		gdImageSetInterpolationMethod(src, GD_BICUBIC);
 	}
 
